#ifndef __LOWI_REQUEST_H__
#define __LOWI_REQUEST_H__

/*====*====*====*====*====*====*====*====*====*====*====*====*====*====*====*

        LOWI Request Interface Header file

GENERAL DESCRIPTION
  This file contains the structure definitions and function prototypes for
  LOWIRequest

Copyright (c) 2012-2013 Qualcomm Atheros, Inc.
  All Rights Reserved.
  Qualcomm Atheros Confidential and Proprietary.

Export of this technology or software is regulated by the U.S. Government.
Diversion contrary to U.S. law prohibited.
=============================================================================*/
#include <inc/lowi_const.h>
#include <inc/lowi_mac_address.h>

namespace qc_loc_fw
{
class LOWIUtils;
/**
 * Base class for all requests LOWI can handle
 */
class LOWIRequest
{
private:
  /** Identity of request originator*/
  char* mRequestOriginator;
  uint32 mRequestId;
public:

  /**
   * Log Tag
   */
  static const char * const TAG;

  /** Type of Request */
  // NOTE:
  // When adding additional requests, update the following utility function:
  //  -- LOWIUtils::to_string(LOWIRequest::eRequestType a)
  //
  //  ADD NEW REQUESTS ONLY AT THE END OF THIS ENUM
  enum eRequestType
  {
    /** Request a passive or active scan from the wifi driver */
    DISCOVERY_SCAN = 0,
    /** Request to do RTT ranging with another wifi node */
    RANGING_SCAN,
    /** Retrieve LOWI capabilities for ranging, etc */
    CAPABILITY,
    /** Clear the WiFi node cached by LOWI */
    RESET_CACHE,
    /** Receive any scan results if and when available */
    ASYNC_DISCOVERY_SCAN_RESULTS,
    /** Do periodic RTT ranging measurements with LOWI managing the
     *  periodic requests to FW */
    PERIODIC_RANGING_SCAN,
    /** Cancel a PERIODIC_RANGING_SCAN request previously sent */
    CANCEL_RANGING_SCAN,
    /** Retrive specific background scan FW capabilities */
    BGSCAN_CAPABILITIES,
    /** Retrieve a list of frequency channels supported by the WiFi driver */
    BGSCAN_CHANNELS_SUPPORTED,
    /** Start a batching periodic background scan */
    BGSCAN_START,
    /** Stop a batching periodic background scan previously requested */
    BGSCAN_STOP,
    /** Retrieve the cached background scan results */
    BGSCAN_CACHED_RESULTS,
    /** Start detection on a "hotlist" of APs provided in this request */
    HOTLIST_SET,
    /** Stop a HOTLIST_SET request sent previously and clear the hotlist */
    HOTLIST_CLEAR,
    /** Start significant change detection on a list of APs provided in
     *  this request */
    SIGNIFINCANT_CHANGE_LIST_SET,
    /** Stop a SIGNIFINCANT_CHANGE_LIST_SET request sent previously and
     *  clear the list of APs */
    SIGNIFINCANT_CHANGE_LIST_CLEAR,
    /** Provides a random MAC OUI for PNO scans */
    SCANNING_MAC_OUI_SET,
    /** Start a batching periodic background scan request on the LP processor */
    BATCHING_START,
    /** Stop a BATCHING_START request previously sent */
    BATCHING_STOP,
    /** Retrieve the cached background scan results from the LP processor */
    BATCHING_CACHED_RESULTS,
    /** Subscribe to receive dynamic capabilities */
    CAPABILITY_SUBSCRIPTION
  };

  /**
   * Constructor
   * @param uint32 Request Id generated by the client
   */
  LOWIRequest (uint32 requestId);
  /** Destructor*/
  virtual ~LOWIRequest () = 0;

  /** Copy Constructor */
  LOWIRequest( const LOWIRequest& rhs );
  /** Assignment operator */
  LOWIRequest& operator=( const LOWIRequest& rhs );

  /**
   * Returns the Request Originator
   * @return char* Request originator's id
   */
  const char* getRequestOriginator () const;

  /**
   * Sets the Request originator for the request.
   * Used internally by LOWI to identify the originator
   * of the request and send the responses back to the originator.
   *
   * Note: Clients of LOWI API do not need to set the originator as
   * there is no affect of doing so.
   */
  void setRequestOriginator (const char* const request_originator);

  /**
   * Returns the RequestId
   * @return RequestId generated by the user of the API
   */
  uint32 getRequestId () const;

  /**
   * Returns the request type
   * @return eRequestType type of request
   */
  virtual eRequestType getRequestType () const = 0;
};

///////////////////////////
// Capability Request
///////////////////////////

/**
 * Capability Request
 */
class LOWICapabilityRequest: public LOWIRequest
{
public:
  /**
   * Constructor
   * @param uint32 Request id generated by the client
   *        This will be echoed back in the corresponding response.
   */
  LOWICapabilityRequest (uint32 requestId);
  /** Destructor*/
  virtual ~LOWICapabilityRequest ();

  /**
   * Returns the request type
   * @return eRequestType type of request
   */
  virtual eRequestType getRequestType () const;
};

//////////////////////////////////
// Capability Subscription Request
//////////////////////////////////

/**
 * Capability Subscription Request
 * Provides the dynamic capabilities as and when they are known.
 * i.e. Client could subscribe to receive LOWI-LP capabilities. Once,
 * LOWI-LP capabilities are known, client will get notification.
 * Subscription is removed after the response is generated or request timeout
 */
class LOWICapabilitySubscriptionRequest: public LOWIRequest
{
private:
  uint32             mTimeoutSec;
public:
  /**
   * Internally calculated based on the time the request
   * was received and the requested expiry
   */
  int64              mTimeoutTimestamp;

  /**
   * Constructor
   * @param uint32 Request id generated by the client
   *        This will be echoed back in the corresponding response.
   * @param uint32 Request timeout in seconds. Request will expire
   *        after the time and LOWI will silently drop the request.
   */
  LOWICapabilitySubscriptionRequest (uint32 requestId, uint32 timeoutSec);
  /** Destructor*/
  virtual ~LOWICapabilitySubscriptionRequest ();

  /**
   * Returns the request type
   * @return eRequestType type of request
   */
  virtual eRequestType getRequestType () const;

  /**
   * Returns the request timeout
   * @return Request time out sec
   */
  virtual uint32 getRequestTimeout() const;
};

///////////////////////////
// Cache Reset Request
///////////////////////////
/**
 * Cache Reset Request
 */
class LOWICacheResetRequest: public LOWIRequest
{
public:
  /**
   * Constructor
   * @param uint32 Request id generated by the client
   *        This will be echoed back in the corresponding response.
   */
  LOWICacheResetRequest (uint32 requestId);
  /** Destructor*/
  virtual ~LOWICacheResetRequest ();

  /**
   * Returns the request type
   * @return eRequestType type of request
   */
  virtual eRequestType getRequestType () const;
};

///////////////////////////
// Discovery scan Request
///////////////////////////
// Forward declaration
class LOWIChannelInfo;

/**
 * Class to make a Discovery scan request
 */
class LOWIDiscoveryScanRequest: public LOWIRequest
{
public:
  /** Defines the available type of Discovery Scan*/
  enum eScanType
  {
    /**Passive Scan*/
    PASSIVE_SCAN,
    /**Active Scan*/
    ACTIVE_SCAN
  };

  /** Defines the available mode of Discovery Scan Request*/
  enum eRequestMode
  {
    /** Request to perform a Fresh scan. The request will trigger a Request
     * being sent to WifiDriver for fresh scan */
    FORCED_FRESH,
    /** Request to perform Fresh scan with throttling. If the last fresh
     * scan results are relatively fresh then this Request will be serviced
     * from Cache. Criteria to determine freshness is
     * current time < last scan time + threshold (defined in config file)
     */
    NORMAL,
    /** Request to return the result from Cache*/
    CACHE_ONLY,
    /** Request to provide the result from Cache but if results
     * are not found in cache then start the fresh scan*/
    CACHE_FALLBACK
  };

  /** Defines the bands for which the Discovery Scan Request can be made */
  enum eBand
  {
    /** 2.4 GHz only*/
    TWO_POINT_FOUR_GHZ,
    /** 5 GHz only */
    FIVE_GHZ,
    /** Both (2.4 & 5 GHz) */
    BAND_ALL
  };

  /**
   * Returns the scan type
   * @return Passive scan or Active scan
   */
  eScanType getScanType () const;
  /**
   * Returns the request mode
   * @return Mode of request
   */
  eRequestMode getRequestMode () const;
  /**
   * Returns Measurement age filter in seconds.
   * For Cache Request, at the time of generating the
   *  response for the request, all the measurements which
   *  are older than Current time - filter age,
   *  will be filtered out from the response.
   *  So '0' age filter could potentially
   *  return no measurements from the cache.
   *
   * For Fresh scan Request, at the time of generating the
   *  response for the request, all the measurements which
   *  are older than Request time to WLAN driver - filter age,
   *  will be filtered out from the response.
   * @return Measurement Age filter in seconds
   */
  uint32 getMeasAgeFilterSec () const;
  /**
   * Returns fallback tolerance in seconds.
   * Only valid when eRequestMode is CACHE_FALLBACK.
   * At the time of evaluating this request, all the
   * requested channels should have last measurement
   * timestamp equal to or greater than Current time -
   * Fallback tolerance. If not, then fallback to
   * trigger a fresh scan. Client can provide 0 if they
   * want to trigger a fresh scan as well.
   * @return Cache fallback tolerance
   */
  uint32 getFallbackToleranceSec () const;
  /**
   * Returns the band
   * @return RF band to scan.
   */
  eBand getBand () const;
  /**
   * Returns the dynamic array containing the ChannelInfo to be scanned.
   * Number of entries in the vector should not be more than
   * MAX_CHAN_INFO_SIZE as that's the max LOWI supports as of
   * now. Any ChannelInfo more than MAX_CHAN_INFO_SIZE will be
   * ignored.
   * @return Dynamic array containing channel specific information
   */
  vector <LOWIChannelInfo> & getChannels ();
  /**
   * Returns absolute timestamp on which the request will be dropped
   * if not processed already. Value should be -1 if the
   * timeoutTimestamp is not valid.
   * @return Request timeout timestamp
   */
  int64 getTimeoutTimestamp () const;
  /**
   * Returns cache request buffer preference.
   * This is set to true if the cache request has to be buffered until the
   * existing transaction is finished.
   * Will be ignored if there is no ongoing transaction.
   * Will be ignored for FRESH_ONLY requestMode
   * This can be used to get the latest results included in the cache request.
   */
  bool getBufferCacheRequest () const;

  /** Destructor*/
  virtual ~LOWIDiscoveryScanRequest ();

  /**
   * Returns the request type
   * @return eRequestType type of the Request
   */
  virtual eRequestType getRequestType () const;

  /**
   * Helper function to create a DiscoveryScanRequest for CACHE_ONLY Request
   * mode. This API could be used to perform CACHE_ONLY discovery scan on any
   * band.
   *
   * @param uint32 Request ID generated by the client to track the response.
   *                Response for this Request will contain the same request Id.
   * @param eBand Band in which the scan is to be performed
   * @param uint32 Measurement age filter in seconds.
   *               For Cache Request, at the time of generating the
   *               response for the request, all the measurements which
   *               are older than Current time - filter age,
   *               will be filtered out from the response.
   *               So '0' age filter could potentially
   *               return no measurements from the cache.
   * @param int64 Timestamp at which this request should be dropped if not
   *              processed already. Should be 0 if this is to be ignored.
   * @param bool The request will be cached and processed after the current
   *             fresh scan completes if this flag is on. Will be honoured only
   *             if there is an ongoing fresh scan at the time of receiving this
   *             request
   */
  static LOWIDiscoveryScanRequest* createCacheOnlyRequest (uint32 requestId,
      eBand band, uint32 meas_age_filter_sec,
      int64 timeoutTimestamp, bool bufferCacheRequest);

  /**
   * Helper function to create a DiscoveryScanRequest for CACHE_FALLBACK
   * Request mode. This API could be used to perform CACHE_ONLY discovery
   * scan on any band.
   *
   * @param uint32 Request ID generated by the client to track the response.
   *                Response for this Request will contain the same request Id.
   * @param eBand Band in which the scan is to be performed
   * @param eScanType Type of discovery scan Active / Passive. Only
   *                  applicable if the request fallsback to trigger a
   *                  fresh scan.
   * @param uint32 Measurement age filter in seconds.
   *               For Cache Request, at the time of generating the
   *               response for the request, all the measurements which
   *               are older than Current time - filter age,
   *               will be filtered out from the response.
   *               So '0' age filter could potentially
   *               return no measurements from the cache.
   *
   *               For Fresh scan Request, at the time of generating the
   *               response for the request, all the measurements which
   *               are older than Request time to WLAN driver - filter age,
   *               will be filtered out from the response.
   * @param uint32 Fallback tolerance in seconds.
   *               At the time of evaluating this request, all the
   *               requested channels should have last measurement
   *               timestamp equal to or greater than Current time -
   *               Fallback tolerance. If not, then fallback to
   *               trigger a fresh scan. Client can provide 0 if they
   *               want to trigger a fresh scan as well.
   * @param int64 Timestamp at which this request should be dropped if not
   *              processed already. Should be 0 if this is to be ignored.
   * @param bool The request will be cached and processed after the current
   *             fresh scan completes if this flag is on. Will be honoured only
   *             if there is an ongoing fresh scan at the time of receiving
   *             this request
   */
  static LOWIDiscoveryScanRequest* createCacheFallbackRequest (
      uint32 requestId,
      eBand band, eScanType type, uint32 meas_age_filter_sec,
      uint32 fallbackToleranceSec,
      int64 timeoutTimestamp, bool bufferCacheRequest);

  /**
   * Helper function to create a DiscoveryScanRequest for NORMAL / FORCED_FRESH
   * Request mode. This API could be used to perform a NORMAL / FORCED_FRESH
   * discovery scan on any band.
   *
   * @param uint32 Request ID generated by the client to track the response.
   *                Response for this Request will contain the same request Id.
   * @param eBand Band in which the scan is to be performed
   * @param eScanType Type of discovery scan Active / Passive.
   * @param uint32 Measurement age filter in seconds.
   *               For Fresh scan Request, at the time of generating the
   *               response for the request, all the measurements which
   *               are older than Request time to WLAN driver - filter age,
   *               will be filtered out from the response.
   * @param int64 Timestamp at which this request should be dropped if not
   *              processed already. Should be 0 if this is to be ignored.
   * @param eRequestMode Mode of request NORMAL / FORCED_FRESH
   */
  static LOWIDiscoveryScanRequest* createFreshScanRequest (uint32 requestId,
      eBand band, eScanType type, uint32 meas_age_filter_sec,
      int64 timeoutTimestamp, eRequestMode mode);

  /**
   * Helper function to create a DiscoveryScanRequest for CACHE_ONLY Request
   * mode. This API could be used to perform CACHE_ONLY discovery scan on
   * specific channels.
   *
   * @param uint32 Request ID generated by the client to track the response.
   *                Response for this Request will contain the same request Id.
   * @param vector <LOWIChannelInfo> Vector should contain the ChannelInfo
   *               of the Channels that need to be scanned.
   *               Maximum channels supported by LOWI right now are 16 and
   *               any thing more than that is ignored.
   * @param uint32 Measurement age filter in seconds.
   *               For Cache Request, at the time of generating the
   *               response for the request, all the measurements which
   *               are older than Current time - filter age,
   *               will be filtered out from the response.
   *               So '0' age filter could potentially
   *               return no measurements from the cache.
   * @param int64 Timestamp at which this request should be dropped if not
   *              processed already. Should be 0 if this is to be ignored.
   * @param bool The request will be cached and processed after the current
   *             fresh scan completes if this flag is on. Will be honoured only
   *             if there is an ongoing fresh scan at the time of receiving this
   *             request
   */
  static LOWIDiscoveryScanRequest* createCacheOnlyRequest (uint32 requestId,
      vector <LOWIChannelInfo>& chanInfo,
      uint32 meas_age_filter_sec, int64 timeoutTimestamp,
      bool bufferCacheRequest);

  /**
   * Helper function to create a DiscoveryScanRequest for CACHE_FALLBACK
   * Request mode. This API could be used to perform CACHE_ONLY discovery
   * scan on specific channels.
   *
   * @param uint32 Request ID generated by the client to track the response.
   *                Response for this Request will contain the same request Id.
   * @param vector <LOWIChannelInfo> Vector should contain the ChannelInfo
   *               of the Channels that need to be scanned.
   *               Maximum channels supported by LOWI right now are 16 and
   *               any thing more than that is ignored.
   * @param eScanType Type of discovery scan Active / Passive. Only
   *                  applicable if the request falls back to trigger a
   *                  fresh scan.
   * @param uint32 Measurement age filter in seconds.
   *               For Cache Request, at the time of generating the
   *               response for the request, all the measurements which
   *               are older than Current time - filter age,
   *               will be filtered out from the response.
   *               So '0' age filter could potentially
   *               return no measurements from the cache.
   *
   *               For Fresh scan Request, at the time of generating the
   *               response for the request, all the measurements which
   *               are older than Request time to WLAN driver - filter age,
   *               will be filtered out from the response.
   * @param uint32 Fallback tolerance in seconds.
   *               At the time of evaluating this request, all the
   *               requested channels should have last measurement
   *               timestamp equal to or greater than Current time -
   *               Fallback tolerance. If not, then fallback to
   *               trigger a fresh scan. Client can provide 0 if they
   *               want to trigger a fresh scan as well.
   * @param int64 Timestamp at which this request should be dropped if not
   *              processed already. Should be 0 if this is to be ignored.
   * @param bool The request will be cached and processed after the current
   *             fresh scan completes if this flag is on. Will be honoured only
   *             if there is an ongoing fresh scan at the time of receiving
   *             this request
   */
  static LOWIDiscoveryScanRequest* createCacheFallbackRequest (
      uint32 requestId,
      vector <LOWIChannelInfo>& chanInfo,
      eScanType type, uint32 meas_age_filter_sec,
      uint32 fallbackToleranceSec,
      int64 timeoutTimestamp, bool bufferCacheRequest);

  /**
   * Helper function to create a DiscoveryScanRequest for NORMAL / FORCED_FRESH
   * Request mode. This API could be used to perform a NORMAL / FORCED_FRESH
   * discovery scan on specific channels.
   *
   * @param uint32 Request ID generated by the client to track the response.
   *                Response for this Request will contain the same request Id.
   * @param vector <LOWIChannelInfo> Vector should contain the ChannelInfo
   *               of the Channels that need to be scanned.
   *               Maximum channels supported by LOWI right now are 16 and
   *               any thing more than that is ignored.
   * @param eScanType Type of discovery scan Active / Passive.
   * @param uint32 Measurement age filter in seconds.
   *               For Fresh scan Request, at the time of generating the
   *               response for the request, all the measurements which
   *               are older than Request time to WLAN driver - filter age,
   *               will be filtered out from the response.
   * @param int64 Timestamp at which this request should be dropped if not
   *              processed already. Should be 0 if this is to be ignored.
   * @param eRequestMode Mode of request NORMAL / FORCED_FRESH
   */
  static LOWIDiscoveryScanRequest* createFreshScanRequest (uint32 requestId,
      vector <LOWIChannelInfo>& chanInfo,
      eScanType type, uint32 meas_age_filter_sec,
      int64 timeoutTimestamp, eRequestMode mode);

private:
  /**
   * Constructor
   * @param uint32 Request id generated by the client
   *        This will be echoed back in the corresponding response.
   */
  LOWIDiscoveryScanRequest (uint32 requestId);

  /** Passive scan or Active scan*/
  eScanType         scanType;
  /** Mode of request*/
  eRequestMode      requestMode;

  /**
   * Measurement age filter in seconds.
   * For Cache Request, at the time of generating the
   *  response for the request, all the measurements which
   *  are older than Current time - filter age,
   *  will be filtered out from the response.
   *  So '0' age filter could potentially
   *  return no measurements from the cache.
   *
   * For Fresh scan Request, at the time of generating the
   *  response for the request, all the measurements which
   *  are older than Request time to WLAN driver - filter age,
   *  will be filtered out from the response.
   */
  uint32             measAgeFilterSec;
  /**
   * Fallback tolerance in seconds.
   * Only valid when eRequestMode is CACHE_FALLBACK.
   * At the time of evaluating this request, all the
   * requested channels should have last measurement
   * timestamp equal to or greater than Current time -
   * Fallback tolerance. If not, then fallback to
   * trigger a fresh scan. Client can provide 0 if they
   * want to trigger a fresh scan as well.
   */
  uint32             fallbackToleranceSec;
  /** RF band to scan.*/
  eBand             band;
  /**
   * Dynamic array containing the ChannelInfo to be scanned.
   * Number of entries in the vector should not be more than
   * MAX_CHAN_INFO_SIZE as that's the max LOWI supports as of
   * now. Any ChannelInfo more than MAX_CHAN_INFO_SIZE will be
   * ignored.
   */
  vector <LOWIChannelInfo> chanInfo;
  /** Absolute timestamp on which the request will be dropped if not processed
   * value should be 0 if the timeoutTimestamp is not valid.
   */
  int64             timeoutTimestamp;
  /** Set this to true if the cache request has to be buffered until the
   * existing transaction is finished.
   * Will be ignored if there is no ongoing transaction.
   * Will be ignored for FRESH_ONLY requestMode
   * This can be used to get the latest results included in the cache request.
   */
  bool           bufferCacheRequest;
  friend class LOWIUtils;
};

/**
 * Defines the info for a Channel
 */
class LOWIChannelInfo
{
private:
  /** Frequency in Mhz*/
  uint32 frequency;
public:
  LOWIChannelInfo ();
  /**
   * Constructor
   * @param uint32 frequency of the channel
   */
  LOWIChannelInfo (uint32 freq);

  /**
   * Constructor
   * @param uint32 Channel number
   * @param LOWIDiscoveryScanRequest::eBand Band of the channel
   */
  LOWIChannelInfo (uint32 channel, LOWIDiscoveryScanRequest::eBand band);

  ~LOWIChannelInfo ();
  /**
   * Returns the band associated with the channel.
   * @return Band for the channel.
   */
  LOWIDiscoveryScanRequest::eBand getBand () const;
  /**
   * Returns the Frequency associated with the channel.
   * @return Frequency of the channel
   */
  uint32 getFrequency () const;
  /**
   * Returns the Channel number for the Channel Info.
   * Channel number corresponds to the Band
   *
   * @return Channel Number
   */
  uint32 getChannel () const;
};


///////////////////////////
// Ranging scan Request
///////////////////////////

/** Defines available type of Wifi Nodes*/
enum eNodeType
{
  NODE_TYPE_UNKNOWN = 0,
  ACCESS_POINT, // access point
  PEER_DEVICE,  // device which is part of a p2p group
  NAN_DEVICE,   // device which is part of a NAN
  STA_DEVICE,   // stand alone station (as in ad-hoc BSS)
  SOFT_AP       // Node in Hotspot mode
};
/** Defines supported RTT types for Ranging scan request */
enum eRttType
{
  RTT1_RANGING = 0,    /* Ranging with no Multipath correction */
  RTT2_RANGING,        /* Ranging with Multipath correction */
  RTT3_RANGING,        /* Two-sided Ranging as defined in 802.11mc spec */
  BEST_EFFORT_RANGING  /* One of the above, which ever the peer supports */
};

/** Defines RTT report types for Ranging scan */
enum eRttReportType
{
  RTT_REPORT_1_FRAME_CFR = 0,  /* Responses sent on a per frame basis with CFR capture */
  RTT_REPORT_1_FRAME_NO_CFR,   /* Responses sent on a per frame basis without CFR captiure */
  RTT_REPORT_AGGREGATE,        /* Single responses containing results for all requested APs and frames*/
  RTT_REPORT_BSSID             /* Responses sent on a per BSSID destination basis */
};

/** Defines the Bandwidth for Ranging scan */
enum eRangingBandwidth
{
  BW_20MHZ = 0,
  BW_40MHZ,
  BW_80MHZ,
  BW_160MHZ
};

/**
 * Defines the information for a Wifi Node
 */
struct LOWINodeInfo
{
  /** MacId of the Node*/
  LOWIMacAddress bssid;
  /** Frequency in Mhz, which the node is transmitting*/
  uint32 frequency;
  /** Either PEER_DEVICE or ACCESS_POINT*/
  eNodeType nodeType;
  /* Spoof MacId needed for peer device ranging.
   * Only valid if nodeType is PEER_DEVICE*/
  LOWIMacAddress spoofMacId;

  /** The Type Of RTT to be performed */
  eRttType rttType;

  /** Bandwidth to be used*/
  eRangingBandwidth bandwidth;

  /**
   * number of packets for each RTT measurement
   * NOTE: maximum of 5 per RTT measurement
   */
  uint8 num_pkts_per_meas;

public:
  /**
   * Constructor
   */
  LOWINodeInfo ();

  /**
   * Validates the NodeInfo
   */
  void validate ();
};

/**
 * Defines the information for a Wifi Node and allows for periodic measurements
 */
struct LOWIPeriodicNodeInfo : public LOWINodeInfo
{
  /**
   * indicates whether request for this WiFi node is periodic or one-shot
   * 0: one-shot, non-zero: periodic
   */
  uint8 periodic;

  /** for periodic requests, this indicates the periodicity of the
   *  measurements (i.e. every 500ms, or every 800ms, etc
   */
  uint32 meas_period;

  /**
   *  for periodic requests, this indicates the number of measurements to be
   *  carried out at the "period" given above
   */
  uint32 num_measurements;

  /**
   * maximum number of times a measurement should be retried if the measurement
   * fails (i.e. it yields no results)
   */
  uint8 num_retries_per_meas;

public:

  /**
   * Constructor
   */
  LOWIPeriodicNodeInfo();

  /**
   * This function checks the parameters for every wifi node in the request and
   * ensures that they are correct. If not, it assigns a valid default parameter
   * so the request can be serviced.
   */
  void validateParams();
};

/**
 * Class to make a Ranging scan request.
 */
class LOWIRangingScanRequest: public LOWIRequest
{
private:
  /**
   * Dynamic array containing the wifi Node Info to be scanned.
   * Currently LOWI only supports 16 wifi nodes to be scanned
   * with Ranging scan. Any thing more than 16 will be ignored.
   */
  vector <LOWINodeInfo> nodeInfo;

  /** Absolute timestamp on which the request will be dropped if not processed
   * value should be 0 if the timeoutTimestamp is not valid.
   */
  int64             timeoutTimestamp;

  /** The Type Of RTT Report the FW should provide */
  eRttReportType rttReportType;

public:

  /**
   * Returns the Dynamic array containing the LOWINodeInfo
   * @return Dynamic array containing the LOWINodeInfo
   */
  vector <LOWINodeInfo> & getNodes ();

  /**
   * Sets the time-out timestamp for the request
   * @return void
   */
  void setTimeoutTimestamp(int64 timestamp);

  /**
   * Returns the timestamp at which the request expires
   * @return timestamp at which the request expires
   */
  int64 getTimeoutTimestamp () const;

  /**
   * Sets the RTT report type for the request
   * @return void
   */
  void setReportType(eRttReportType report_type);

  /**
   * Returns the RTT Report Type for this Request
   * @return eRttReportType RTT Report type
   */
  eRttReportType getReportType() const;

  /**
   * Constructor
   * @param uint32 Request id generated by the client
   *        This will be echoed back in the corresponding response.
   * @param vector<LOWINodeInfo> Dynamic array containing wifi nodes
   *                           that need to be scanned.
   * @param int64 Timestamp at which the request will be dropped if not
   *              processed already. Should be 0 if this is to be ignored.
   */
  LOWIRangingScanRequest (uint32 requestId, vector <LOWINodeInfo>& node_info,
      int64 timestamp);
  /** Destructor*/
  virtual ~LOWIRangingScanRequest ();

  /**
   * Returns the request type
   * @return eRequestType type of the Request
   */
  virtual eRequestType getRequestType () const;
};

/**
 * Class to make a Request for Async discovery scan results.
*/
class LOWIAsyncDiscoveryScanResultRequest : public LOWIRequest
{
private:
  /**
   * Number of seconds for which the request should be active.
   * Request will be dropped after this time.
   * requestExpirySec can not be more than
   * ASYNC_DISCOVERY_REQ_VALIDITY_PERIOD_SEC
   */
  uint32             requestExpirySec;

  /**
   * Internally calculated based on the time the request
   * was received and the requested expiry
   */
  int64              timeoutTimestamp;

public:
  /**
   * Constructor
   * A client registers this request to receive the discovery
   * scan results as and when they are available.
   * LOWI provides the async results to the client for as long
   * as the request is valid or for a max period
   * (ASYNC_DISCOVERY_REQ_VALIDITY_PERIOD_SEC), after which it
   * drops the request assuming that the client is no longer
   * interested in the scan results. There is no indication to
   * the client when this happens. If the client is interested
   * in listening for the async scan results, it should keep
   * refreshing it's registration by issung the same request
   * before the expiry of previous request.
   * Issuing the same request again even when it's not yet expired,
   * just refreshes the request and LOWI resets the expiry timer
   * for the request.
   * If the client is no longer interested in listening for the
   * async results, it can refresh this request with 0 timeout value.
   *
   * Client will not get the results / response to this request
   * if there was a discovery scan result available but was
   * triggered by a discovery scan request from the same client.
   * @param uint32 Request id generated by the client
   *        This will be echoed back in the corresponding response.
   * @param uint32 Number of seconds for which the request should
   *               be active. requestExpirySec can not be more than
   *               ASYNC_DISCOVERY_REQ_VALIDITY_PERIOD_SEC
   */
  LOWIAsyncDiscoveryScanResultRequest (uint32 requestId,
      uint32 request_timer_sec);

  /**
   * Returns the expiry time of the Request.
   * @return uint32 Request expiry time
   */
  uint32 getRequestExpiryTime ();

  /**
   * Returns the request type
   * @return eRequestType type of the Request
   */
  virtual eRequestType getRequestType () const;

  /**
   * Returns the Timeout timestamp
   * @return int64 Timeout timestamp
   */
  int64 getTimeoutTimestamp () const;

  /**
   * Sets the timeout timestamp
   * Client does not have to call this API.
   * Calling this API from client side does not affect
   * the internal usage.
   * This is internally calculated based on the
   * request timer in the constructor.
   * @param int64 Timeout timestamp
   */
  void setTimeoutTimestamp (int64 timeout);
};


/**
 * Class to make a periodic ranging scan request.
 */
  class LOWIPeriodicRangingScanRequest: public LOWIRangingScanRequest
{
private:
  /**
   * Dynamic array containing the wifi Node Info to be scanned.
   * Currently LOWI only supports 16 wifi nodes to be scanned
   * with Ranging scan. Any thing more than 16 will be ignored.
   */
  vector <LOWIPeriodicNodeInfo> nodeInfo;

protected:
  /**
   * Use the empty vector "nodes" to pass to the LOWIRangingScanRequest
   * constructor when creating a LOWIPeriodicRangingScanRequest type. That way,
   * LOWIPeriodicRangingScanRequest types can be created using the
   * already existing constructors.
   */
  static vector<LOWINodeInfo> emptyNodeInfo;


public:
  /**
   * Returns the Dynamic array containing the LOWINodeInfo
   * @return Dynamic array containing the LOWINodeInfo
   */
  vector <LOWIPeriodicNodeInfo> & getNodes ();

  /**
   * Constructor
   * @param uint32 Request id generated by the client
   *        This will be echoed back in the corresponding response.
   * @param vector<LOWIPeriodicNodeInfo> Dynamic array containing wifi nodes
   *                           that need to be scanned.
   * @param int64 Timestamp at which the request will be dropped if not
   *              processed already. Should be 0 if this is to be ignored.
   */
  LOWIPeriodicRangingScanRequest (uint32 requestId,
                                  vector <LOWIPeriodicNodeInfo>& node_info,
                                  int64 timestamp);
  /** Destructor*/
  virtual ~LOWIPeriodicRangingScanRequest ();

  /**
   * Returns the request type
   * @return eRequestType type of the Request
   */
  virtual eRequestType getRequestType () const;
};

/**
 * This class cancels RTT measurements
 */
class LOWICancelRangingScanRequest: public LOWIRequest
{
private:
  /**
   * Dynamic array containing the wifi node bssids to be
   * cancelled
   */
  vector <LOWIMacAddress> bssidInfo;

public:
  /**
   * Constructor
   * @param uint32 Request id generated by the client
   *        This will be echoed back in the corresponding response.
   * @param vector<LOWIMacAddress> Dynamic array containing wifi nodes
   *                           that need to be cancelled.
   */
  LOWICancelRangingScanRequest (uint32 requestId,
                                vector<LOWIMacAddress> & bssid_info);

  /** Destructor*/
  virtual ~LOWICancelRangingScanRequest();

  /**
   * Returns the Dynamic array containing the BSSIDs of the wifi nodes to cancel
   * @return Dynamic array containing the BSSIDs
   */
  vector <LOWIMacAddress> & getBssids();

  /**
   * Returns the request type
   * @return eRequestType type of the Request
   */
  virtual eRequestType getRequestType() const;
};


/////////////////////////////////////////
// Gscan scan definitions and classes
/////////////////////////////////////////

/** Get background scan capabilities request */
class LOWIGscanCapabilitiesRequest : public LOWIRequest
{
public:
  /**
   * Constructor
   * @param requestId generated by the client
   *        This will be echoed back in the corresponding response.
   */
  LOWIGscanCapabilitiesRequest(uint32 requestId);
  ~LOWIGscanCapabilitiesRequest();

  /**
   * Returns the request type
   * @return eRequestType type of request
   */
  virtual eRequestType getRequestType() const;
};

/** Channels supported request */
class LOWIChannelsSupportedRequest : public LOWIRequest
{
private:
  int32 mBand;
  int32 mMaxChannels;

public:
  /**
   * Constructor
   * @param requestId generated by the client
   *        This will be echoed back in the corresponding response.
   * @param band: channels supported in this band (2G or 5G or both)
   * @param maxChannels: max number of channels to be returned in the response
   */
  LOWIChannelsSupportedRequest(uint32 requestId, int32 band, int32 maxChannels);
  ~LOWIChannelsSupportedRequest();

  /**
   * Returns the band requested
   * @return int32 band requested
   */
  virtual int32 getBand() const;

  /**
   * Returns the maximum number of channels to be returned in the response
   * @return int32 maximum number of channels to be returned in the response
   */
  virtual int32 getMaxChannels() const;

  /**
   * Returns the request type
   * @return eRequestType type of request
   */
  virtual eRequestType getRequestType() const;
};

/** Defines the bands for which the Background Scan Request can be made */
typedef enum
{
  /** Unspecified frequency band */
  BAND_UNSPECIFIED,
  /** 2.4 GHz only  */
  BAND_BG = 1,
  /** 5 GHz only, excluding DFS channels */
  BAND_A = 2,
  /** 5 GHz DFS only channels */
  BAND_A_DFS = 4,
  /** 5 GHz only, including DFS channels */
  BAND_A_WITH_DFS = 6,
  /** Both (2.4 & 5 GHz), excluding DFS channels */
  BAND_ABG = 3,
  /** Both (2.4 & 5 GHz), including DFS channels */
  BAND_ABG_WITH_DFS = 7,
} eGscanBand;

/** Defines channel information used by FW to scan the channel */
struct LOWIScanChannelInfo
{
  /** 20MHz primary center frequency */
  uint32 freq;
  /** dwell time (hint) */
  uint32 dwellTimeMs;
  /** 0 => active, 1 => passive scan; ignored for DFS */
  uint8  scanType;
};

/** Defines bucket information */
struct LOWIScanBucketInfo
{
  /** bucket index, 0 based */
  uint32     bucketIdx;
  /** band to scan, when UNSPECIFIED, use channel list below */
  eGscanBand band;
  /** scan period in ms, if too low, FW can choose a suitable period
   *  that will generate results as fast as possible instead of
   *  rejecting the command
   */
  uint32     period;
  /** report_events semantics -
   *  0 => report only when scan history is % full
   *  1 => same as 0 + report a scan completion event after scanning this bucket
   *  2 => same as 1 + forward scan results (beacons/probe responses + IEs) in real time to HAL
   *  3 => same as 2 + forward scan results (beacons/probe responses + IEs) in real time to
   *       supplicant as well (optional) .
   */
  uint8      reportEvents;
  /** number of channels to scan in this bucket */
  uint8      numChannels;
  /** list of channels to scan in this bucket */
  LOWIScanChannelInfo channels[LOWI_MAX_CHANNELS_GSCAN];
};

/**
 * Defines the background scan parameters
 */
struct LOWIScanParams
{
  /** Base timer period in ms. All bucket periods must be a multiple of
   *  this base timer period.
   */
  uint32 basePeriod;
  /** Number of APs to store in each scan in the BSSID/RSSI history buffer in
   *  FW. The criterion for storage is highest RSSI.
   */
  uint32 maxApPerScan;
  /** Percentage threshold at which SCAN_BUFFER_FULL event shall be
   *  generated by the FW
   */
  uint8 reportThresh;
  /** number of buckets in this particular request */
  uint8 numBuckets;
  /** array containing the specs for each bucket */
  LOWIScanBucketInfo buckets[LOWI_MAX_BUCKETS];
};

/** Gscan scan start request */
class LOWIBgscanStartRequest : public LOWIRequest
{
private:
  /** Scan parameters to be used by the FW */
  LOWIScanParams mScanParams;

public:
  /**
   * Constructor
   * @param requestId: unique request identifier
   * @param scanParams: scan parameters
   */
  LOWIBgscanStartRequest(uint32 requestId, LOWIScanParams &scanParams);

  /** Destructor*/
  virtual ~LOWIBgscanStartRequest();

  /**
   * Returns the scan parameters
   * @return LOWIScanParams&: scan parameters to be used by FW to perform the
   *         background scan
   */
  LOWIScanParams& getScanParams();

  /**
   * Returns the request type
   * @return eRequestType type of the Request
   */
  virtual eRequestType getRequestType () const;
};

/** Background scan stop request */
class LOWIBgscanStopRequest : public LOWIRequest
{
public:
  /**
   * Constructor
   * @param requestId: unique request identifier
   */
  LOWIBgscanStopRequest (uint32 requestId);

  /** Destructor*/
  virtual ~LOWIBgscanStopRequest();

  /**
   * Returns the request type
   * @return eRequestType type of the Request
   */
  virtual eRequestType getRequestType () const;
};

/** Get cached bgscan results request */
class LOWIGetCachedBgscanResultsRequest : public LOWIRequest
{
private:
  /** Flag indicating whether to flush the results buffer after retrieving
   *  the results
   */
  int8 mFlush;

  /** Maximum number of scan results to retrieve from the FW */
  uint32 mMaxResultsToRetrieve;

public:
   /**
    * Constructor
    * @param requestId: unique request identifier
    * @param flush : indicates whether to flush the cache after retrieving the
    *             results.  0: FW not to flush the cache after returning
    *             results, 1: FW to flush the cache after returning results
    * @param maxResults: maximum number of results to be returned to client
    */
  LOWIGetCachedBgscanResultsRequest(uint32 requestId, int8 flush, uint32 maxResults);
  /** Destructor */
  ~LOWIGetCachedBgscanResultsRequest();

  /**
   * Retrieve flag that indicates whether to flush the cache or not after
   * retrieving the results
   * @return int8
   */
  int8 getFlush() const;

  /**
   * Return the maximum number of results to be returned to client
   * @return uint32
   */
  uint32 getMaxResultsToRetrieve() const;

  /**
   * Returns the request type
   * @return eRequestType type of request
   */
  virtual eRequestType getRequestType() const;
};

/** Defines the detection parameters for a given AP in an AP hotlist */
struct LOWIApThreshParams
{
  /** MAC address */
  LOWIMacAddress bssid;
  /** Low RSSI detection threshold */
  int16 low;
  /** High RSSI detection threshold */
  int16 high;
  /** 20MHz primary channel frequency (hint)  */
  uint32 channel;
};

/** Defines the hotlist information */
struct LOWIHotlistParams
{
  /** Number of samples to confirm AP loss */
  uint32 lostApSampleSize;
  /** Number of APs in the hotlist */
  uint32 numAps;
  /** Hotlist of APs */
  LOWIApThreshParams ap[LOWI_MAX_HOTLIST_APS];
};

/** BSSID Hotlist set request */
class LOWIHotlistSetRequest : public LOWIRequest
{
private:
  /** hotlist information to be used by the FW */
  LOWIHotlistParams mHotlistParams;

public:

  /**
   * Constructor
   * @param requestId: unique request identifier
   * @param params: list of APs to be monitored
   */
  LOWIHotlistSetRequest (uint32 requestId, LOWIHotlistParams &params);

  /** Destructor*/
  virtual ~LOWIHotlistSetRequest();

  /**
   * Returns the request type
   * @return eRequestType type of the Request
   */
  virtual eRequestType getRequestType () const;

  /**
   * Returns the hotlist information
   * @return LOWIHotlistParams&
   */
  LOWIHotlistParams &getHotlist();

};

/** BSSID Hotlist clear request */
class LOWIHotlistClearRequest : public LOWIRequest
{
public:
  /**
   * Constructor
   * @param requestId: unique request identifier
   */
  LOWIHotlistClearRequest (uint32 requestId);

  /** Destructor*/
  virtual ~LOWIHotlistClearRequest();

  /**
   * Returns the request type
   * @return eRequestType type of the Request
   */
  virtual eRequestType getRequestType () const;
};

/** Defines the significant change list parameters */
struct LOWISignificantChangeParams
{
  /** number of samples for averaging RSSI */
  uint32 rssiSampleSize;
  /** number of samples to confirm AP loss */
  uint32 lostApSampleSize;
  /** number of APs breaching threshold */
  uint32 minBreaching;
  /** number of APs (max 64) */
  uint32 numAps;
  /** list of significant change APs   */
  LOWIApThreshParams ap[LOWI_MAX_SIGNIFICANT_CHANGE_APS];
};

/** Significant change list set request */
class LOWISignificantChangeListSetRequest : public LOWIRequest
{
private:
  LOWISignificantChangeParams mSigChangeParams;

public:
  /**
   * Constructor
   * @param requestId: unique request identifier
   * @param params: significant change list parameters
   */
  LOWISignificantChangeListSetRequest (uint32 requestId,
                                       LOWISignificantChangeParams &params);

  /** Destructor*/
  virtual ~LOWISignificantChangeListSetRequest();

  /**
   * Returns the hotlist information
   * @return LOWIHotlistParams&
   */
  LOWISignificantChangeParams &getSigChangeParams();

  /**
   * Returns the request type
   * @return eRequestType type of request
   */
  virtual eRequestType getRequestType() const;
};

/** Significant change list clear request */
class LOWISignificantChangeListClearRequest : public LOWIRequest
{
public:
  /**
   * Constructor
   * @param requestId: unique request identifier
   */
  LOWISignificantChangeListClearRequest (uint32 requestId);

  /** Destructor*/
  virtual ~LOWISignificantChangeListClearRequest();

  /**
   * Returns the request type
   * @return eRequestType type of request
   */
  virtual eRequestType getRequestType() const;
};


/** set scanning MAC OUI request */
class LOWIScanningMacOuiSetRequest : public LOWIRequest
{
private:
  /** Scanning OUI to be used by FW */
  uint8 mOui[3];

public:
  /**
   * Constructor
   * @param requestId: unique request identifier
   * @param scanOui: OUI for FW to scan
   */
  LOWIScanningMacOuiSetRequest (uint32 requestId, uint8* scanOui);

  /** Destructor*/
  virtual ~LOWIScanningMacOuiSetRequest();

  /**
   * Return the scanning OUI
   * @return uint8*: ptr to OUI array
   */
  uint8* getOui();

  /**
   * Returns the request type
   * @return eRequestType type of request
   */
  virtual eRequestType getRequestType() const;
};

/** Batching scan start request from LP processor */
class LOWIBatchingStartRequest : public LOWIBgscanStartRequest
{
public:
  /**
   * Constructor
   * @param requestId: unique request identifier
   * @param scanParams: scan parameters
   *
   * NOTE:
   * Client only needs to provide the reportThresh in the LOWIScanParams for
   * this request
   */
  LOWIBatchingStartRequest(uint32 requestId, LOWIScanParams &scanParams);

  /** Destructor*/
  virtual ~LOWIBatchingStartRequest();

  /**
   * Returns the request type
   * @return eRequestType type of the Request
   */
  virtual eRequestType getRequestType () const;
};

/** Batching scan stop request from LP processor */
class LOWIBatchingStopRequest : public LOWIBgscanStopRequest
{
public:
  /**
   * Constructor
   * @param requestId: unique request identifier
   */
  LOWIBatchingStopRequest (uint32 requestId);

  /** Destructor  */
  virtual ~LOWIBatchingStopRequest();

  /**
   * Returns the request type
   * @return eRequestType type of the Request
   */
  virtual eRequestType getRequestType () const;
};

/** Get cached bgscan results from LP processor */
class LOWIGetBatchingResultsRequest : public LOWIGetCachedBgscanResultsRequest
{
public:
  /**
   * Constructor
   * @param requestId: unique request identifier
   * @param flush : indicates whether to flush the cache after retrieving the
   *             results.
   * @param maxResults: maximum number of results to be returned to client
   */
  LOWIGetBatchingResultsRequest(uint32 requestId, int8 flush, uint32 maxResults);

  /** Destructor */
  ~LOWIGetBatchingResultsRequest();

  /**
   * Returns the request type
   * @return eRequestType type of request
   */
  virtual eRequestType getRequestType() const;
};
}// namespace qc_loc_fw

#endif //#ifndef __LOWI_REQUEST_H__
