#ifndef __LOWI_SCAN_MEASUREMENT_H__
#define __LOWI_SCAN_MEASUREMENT_H__

/*====*====*====*====*====*====*====*====*====*====*====*====*====*====*====*

        LOWI Scan Measurement Interface Header file

GENERAL DESCRIPTION
  This file contains the structure definitions and function prototypes for
  LOWIScanMeasurement

Copyright (c) 2012-2013 Qualcomm Atheros, Inc.
  All Rights Reserved.
  Qualcomm Atheros Confidential and Proprietary.

Export of this technology or software is regulated by the U.S. Government.
Diversion contrary to U.S. law prohibited.
=============================================================================*/
#include <inc/lowi_request.h>
#include <inc/lowi_ssid.h>
#include <inc/lowi_mac_address.h>

namespace qc_loc_fw
{

/**
 * This struct defines the MSAP related data.
 */
struct LOWIMsapInfo
{
  uint8 protocolVersion;
  uint32        venueHash;
  uint8 serverIdx;
};

/**
 * This struct defines Measurement Info per Wifi Node.
 * For a DiscoveryScan only rssi and rssi_timestamp are valid fields
 * For a Ranging scan rtt, rtt_timestamp and rssi are valid fields.
 * 5/30/2014: for ranging scans that use RTT3 additional parameters
 * have been added: preamble, bw, mcsIdx and bitrate
 */
struct LOWIMeasurementInfo
{
  /**
   * Measurement age msec
   * In units of 1 msec, -1 means info not available
   */
  int32         meas_age;
  /** RTT - value in nsec. 0 is considered to be invalid rtt*/
  int32         rtt;
  /* For RangingScanResponse
   * time stamp is an averaged time stamp, if there are multiple
   * measurements for a Wifi Node.
   */
  int64         rtt_timestamp;
  /** Measurement time stamp.
   * For DiscoveryScanResponse
   *  time stamp is corresponding to when the beacon was received.
   */
  int64         rssi_timestamp;
  /** Signal strength in 0.5dBm*/
  int16         rssi;
  int16         reserved1;
  /** preamble */
  int8          preamble;
  /** bandwidth */
  int8          bw;
  /** Modulation Coding Scheme (MCS) index defines the
   *  following:
   *  1) The Number of Streams used for TX & RX
   *  2) the Type of mudulation used
   *  3) the Coding Rate
   *
   *  Note: Does not apply to legacy frames (frames using schemes
   *  prior to 802.11n)
   *   */
  int8          mcsIdx; /* does not apply to OFDM preambles */
  int8          reserved2;
  /** bitrate */
  int32         bitrate; /* in units of 100Kbps */
  int32         reserved3;
public:
  /** constructor */
  LOWIMeasurementInfo();
};

/**
 * This class defines the measurement taken for every scan request.
 * This contains the measurements corresponding the discovery / ranging
 * and background scan requests. However, the fields are valid /
 * invalid based on type of scan as documented below.
 */
class LOWIScanMeasurement
{
public:
  /** Log Tag */
  static const char * const TAG;

  /** BSSID of the wifi node*/
  LOWIMacAddress  bssid;

  /** Operating Channel Information */
  uint32          frequency;         /* Primary Channel Frequency */

  /************* INTERNAL FIELDS *******************
   *  The following fields are used internaly within LOWI and
   *  will not be provided to Clients. Clients shall not rely
   *  on the values of these fields.
   */
  /** Operating Channel Information... Contd */
  uint32          band_center_freq1; /* Channel Center Frequency for whole BW (used for 40, 80 & 160MHz BW) */
  uint32          band_center_freq2; /* Channel Center frequency of second 80 MHz lobe for 80 + 80 BW */
  /** The following info field is a bitfield that contains PHY
    * mode and flags for this bssid.
    * Bits 0-6 contain the PHY mode
    * Bits 7-13 contain the flags */
  uint32          info;
  /* Indicates if the BSSID supports 11mc - dual sided RTT */
  uint32          ranging_features_supported;
  /************* END INTERNAL FIELDS ***************/

  /** Secure access point or not. Only valid for DiscoveryScanResponse*/
  bool            isSecure;
  /** Type of the Wifi Node. Only valid for DiscoveryScanResponse*/
  eNodeType       type;
  /** Type of RTT measurement performed, if applicable */
  eRttType        rttType;
  /** SSID. Only valid for DiscoveryScanResponse*/
  LOWISsid        ssid;
  /** MsapInfo - valid if not NULL. Only valid for DiscoveryScanResponse*/
  LOWIMsapInfo*   msapInfo;
  /** Cell power limit in dBm. Only valid for discovery scan results,
   *  if available. For ranging scan results will be always 0.
   */
  int8            cellPowerLimitdBm;

  /**
   * Indicates if AP is indoor or outdoor
   * ' ' indicates - Do not care
   * 'I' indicates - Indoor
   * 'O' indicates - Outdoor
   */
  uint8         indoor_outdoor;

  /**
   * Dynamic array containing measurement info per Wifi node.
   * DiscoveryScan and background scans will only have one measurement
   * where as the the vector can contain multiple MeasurementInfo for a
   * Ranging scan.
   */
  vector <LOWIMeasurementInfo*> measurementsInfo;

  /** Contains the Country Code
   * i.e. 'U' 'S'
   * if there is no country code found, then the array will contain 0 0
   */
  uint8         country_code [LOWI_COUNTRY_CODE_LEN];

  /**
   * Measurement number. In case of periodic ranging scan measurements, this
   * will provide a counter that the client can use to track the number of
   * measurements at any given point during the ranging request. It does not
   * apply to single-shot ranging requests. For single-shot requests this will
   * always be zero.
   */
  uint32 measurementNum;

  /** The following four params
   *  -- beaconPeriod
   *  -- beaconCaps
   *  -- ieLen
   *  -- ieData
   *  are part of the results for background scans
   */

  /** Period advertised in the beacon */
  uint16 beaconPeriod;

  /** Capabilities advertised in the beacon */
  uint16 beaconCaps;

  /** Blob of all the information elements found in the beacon */
  vector<int8> ieData;

  /** Constructor*/
  LOWIScanMeasurement ();
  /** Destructor*/
  ~LOWIScanMeasurement ();

  /** Copy constructor **/
  LOWIScanMeasurement( const LOWIScanMeasurement& rhs );
  /** Assignment operator **/
  LOWIScanMeasurement& operator=( const LOWIScanMeasurement& rhs );
};

} // namespace qc_loc_fw

#endif //#ifndef __LOWI_SCAN_MEASUREMENT_H__
